; initialization file for XLISP

(unless (fboundp 'strcat) ; backwards compatibility if COMMONLISP defined
	(defmacro strcat (&rest str) (concatenate 'string ,@str)))

(defun init1() 
          (wdef 4 1 480 250 16 2)
          (setq *debug-io* *standard-input*)
          (setq *error-output* *standard-output*)
          (setq *trace-output* *standard-output*)
          (cls)
)

(defun init2()                ; still needs some hacking
          (wdef 4 1 480 200 16 2)
          (cls)
          (setq *debug-io* (open "con_" :direction :io :if-exists :overwrite))
          (wdef *debug-io* 4 1 480 50 16 204)
          (setq *error-output* *debug-io*)
          (setq *trace-output* *debug-io*)
          (cls *debug-io*)
)

(setq *float-format* "%.8g")

(setf (symbol-function 'pp) #'pprint)
(defmacro pp-def (exp)
     (pprint (get-lambda-expression
                     (symbol-function ',exp))))

(init1)

; (makunbound sym) - make a symbol value be unbound
(defun makunbound (sym) (setf (symbol-value sym) '*unbound*) sym)

; (fmakunbound sym) - make a symbol function be unbound
(defun fmakunbound (sym) (setf (symbol-function sym) '*unbound*) sym)

; (mapcan fun list [ list ]...)
(defmacro mapcan (&rest args) (apply #'nconc (mapcar ,@args)))

; (mapcon fun list [ list ]...)
(defmacro mapcon (&rest args) (apply #'nconc (maplist ,@args)))

; (set-macro-character ch fun [ tflag ])
(defun set-macro-character (ch fun &optional tflag)
    (setf (aref *readtable* (char-int ch))
          (cons (if tflag :tmacro :nmacro) fun))
    t)

; (get-macro-character ch)
(defun get-macro-character (ch)
  (if (consp (aref *readtable* (char-int ch)))
    (cdr (aref *readtable* (char-int ch)))
    nil))

; (savefun fun) - save a function definition to a file
(defmacro savefun (fun)
  (let* ((fname (strcat (symbol-name ',fun) ".lsp"))
          (fval (get-lambda-expression (symbol-function ',fun)))
          (fp (open fname :direction :output)))
     (cond (fp (pprint (cons (if (eq (car fval) 'lambda)
                                 'defun
                                 'defmacro)
                            (cons ',fun (cdr fval))) fp)
               (close fp)
               fname)
           (t nil))))

; (debug) - enable debug breaks
(defun debug ()
       (setq *breakenable* t))

; (nodebug) - disable debug breaks
(defun nodebug ()
       (setq *breakenable* nil))

; initialize to enable breaks but no trace back
(setq *breakenable* t)
(setq *tracenable*  t)
(setq *tracelimit*  50)

(defmacro step (&rest args)
  (load 'stepper)
  (step ,@args)
)
